import { BatchWriteItemCommand, DeleteItemCommand, PutItemCommand, UpdateItemCommand } from '@aws-sdk/client-dynamodb';
import { unmarshall } from '@aws-sdk/util-dynamodb';
import { DDBReadOnlyClient } from './DDBReadOnlyClient';
import { mapUpdateItemCommand } from './mapper/mapUpdateItemCommand';
import { mapDeleteItemsCommand } from './mapper/mapDeleteItemsCommand';
import { mapDeleteItemCommand } from './mapper/mapDeleteItemCommand';
import { mapCreateItemsCommand } from './mapper/mapCreateItemsCommand';
import { mapCreateItemCommand } from './mapper/mapCreateItemCommand';
import { formatKey } from './formatter/formatKey';
export class DDBClient extends DDBReadOnlyClient {
    constructor(configuration) {
        super(configuration);
    }
    async createItem(item) {
        let command = undefined;
        try {
            const partitionKey = item[this.config.partitionKeyName];
            const sortKey = this.config.sortKeyName ? item[this.config.sortKeyName] : undefined;
            this.requireFullPrimaryKey(partitionKey, sortKey);
            command = mapCreateItemCommand(this.config, item);
            await this.client.send(new PutItemCommand(command));
            return item;
        }
        catch (err) {
            console.error('Unable to create item:', JSON.stringify(item), 'with command', JSON.stringify(command));
            throw new Error(err);
        }
    }
    async createItems(items) {
        let command = undefined;
        try {
            command = mapCreateItemsCommand(this.config, items);
            await this.client.send(new BatchWriteItemCommand(command));
        }
        catch (err) {
            console.error('Unable to create items:', JSON.stringify(items), 'with command', JSON.stringify(command));
            throw new Error(err);
        }
    }
    async updateItem(item, createIfNotExists = false) {
        const partitionKey = item[this.config.partitionKeyName];
        const sortKey = this.config.sortKeyName && item[this.config.sortKeyName];
        let command = undefined;
        try {
            this.requireFullPrimaryKey(partitionKey, sortKey);
            command = mapUpdateItemCommand(this.config, item, createIfNotExists);
            const response = await this.client.send(new UpdateItemCommand(command));
            if (!response.Attributes)
                throw new Error('Updated item not returned');
            return unmarshall(response.Attributes);
        }
        catch (error) {
            if (error?.name === 'ConditionalCheckFailedException') {
                throw new Error(`Can't update item as it does not exist: ${formatKey(this.config, partitionKey, sortKey)}`);
            }
            console.error('Unable to update item:', JSON.stringify(item), 'with command', JSON.stringify(command));
            throw new Error(error);
        }
    }
    async deleteItem(partitionKey, sortKey) {
        let command = undefined;
        try {
            this.requireFullPrimaryKey(partitionKey, sortKey);
            command = mapDeleteItemCommand(this.config, partitionKey, sortKey);
            await this.client.send(new DeleteItemCommand(command));
        }
        catch (err) {
            console.error('Unable to delete item:', formatKey(this.config, partitionKey, sortKey), 'with command', JSON.stringify(command));
            throw new Error(err);
        }
    }
    async deleteItems(items) {
        let command = undefined;
        try {
            if (items.length === 0)
                return;
            command = mapDeleteItemsCommand(this.config, items);
            await this.client.send(new BatchWriteItemCommand(command));
        }
        catch (err) {
            console.error('Unable to delete items:', JSON.stringify(items), 'with command', JSON.stringify(command));
            throw new Error(err);
        }
    }
}
//# sourceMappingURL=DDBClient.js.map