import { DynamoDBClient, GetItemCommand, QueryCommand } from '@aws-sdk/client-dynamodb';
import { unmarshall } from '@aws-sdk/util-dynamodb';
import { mapGetItemCommand } from './mapper/mapGetItemCommand';
import { mapGetItemsCommand } from './mapper/mapGetItemsCommand';
import { mapGetItemIndexCommand } from './mapper/mapGetItemIndexCommand';
import { formatKey } from './formatter/formatKey';
export class DDBReadOnlyClient {
    config;
    client;
    constructor(config) {
        this.config = config;
        this.client = new DynamoDBClient(config.configuration || {});
    }
    async getItem(partitionKey, sortKey) {
        let command = undefined;
        try {
            this.requireFullPrimaryKey(partitionKey, sortKey);
            if (!this.config.indexName) {
                command = mapGetItemCommand(this.config, partitionKey, sortKey);
                const response = await this.client.send(new GetItemCommand(command));
                return response?.Item ? unmarshall(response?.Item) : null;
            }
            else {
                command = mapGetItemIndexCommand(this.config, partitionKey, sortKey);
                const response = await this.client.send(new QueryCommand(command));
                const items = this.extractItems(response);
                return items.length > 0 ? items[0] : null;
            }
        }
        catch (err) {
            console.error('Unable to get item:', formatKey(this.config, partitionKey, sortKey), 'with command', JSON.stringify(command));
            throw new Error(err);
        }
    }
    async getItems(partitionKey, sortKeyPrefix) {
        let command = undefined;
        try {
            command = mapGetItemsCommand(this.config, partitionKey, sortKeyPrefix);
            const response = await this.client.send(new QueryCommand(command));
            return this.extractItems(response);
        }
        catch (err) {
            console.error('Unable to get items:', formatKey(this.config, partitionKey, sortKeyPrefix), 'with command', JSON.stringify(command));
            throw new Error(err);
        }
    }
    extractItems(response) {
        return response?.Items?.map(item => unmarshall(item)) || [];
    }
    requireFullPrimaryKey(partitionKey, sortKey) {
        if (!partitionKey)
            throw new Error(`Missing partitionKey.`);
        if (this.config.sortKeyName && !sortKey)
            throw new Error(`Missing sortKey.`);
    }
}
//# sourceMappingURL=DDBReadOnlyClient.js.map